import React, { FormEvent, PureComponent } from 'react';
import { hot } from 'react-hot-loader';
import { connect } from 'react-redux';
import { NavModel } from '@grafana/data';
import { StoreState } from 'app/types';
import { updateLocation } from 'app/core/reducers/location';
import { getNavModel } from 'app/core/selectors/navModel';
import { getLocationQuery } from 'app/core/selectors/location';
import Page from 'app/core/components/Page/Page';
import { UpgradeInfo } from 'app/features/admin/UpgradePage';
import { LicenseChrome } from 'app/features/admin/LicenseChrome';
import { getLicenseToken, postLicenseToken, LicenseToken, VALID, EXPIRED } from './state/api';
import { initLicenseWarnings, renderLicenseWarning } from './index';
import { LicenseTokenUpload } from './LicenseTokenUpload';

interface OwnProps {
  navModel: NavModel;
  getLicenseToken: typeof getLicenseToken;
  postLicenseToken: typeof postLicenseToken;
  tokenUpdated?: boolean;
}

interface DispatchProps {
  updateLocation: typeof updateLocation;
}

type Props = OwnProps & DispatchProps;

interface State {
  token: LicenseToken;
  isLoading: boolean;
  isUploading: boolean;
}

export class LicensePage extends PureComponent<Props, State> {
  constructor(props: Props) {
    super(props);

    this.state = {
      token: null,
      isLoading: true,
      isUploading: false,
    };
  }

  async componentDidMount() {
    renderLicenseWarning();
    const token = await this.props.getLicenseToken().catch(err => null);
    this.setState({ token, isLoading: false });
  }

  async componentWillUnmount() {
    initLicenseWarnings();
  }

  onFileUpload = (event: FormEvent<HTMLInputElement>) => {
    const file = event.currentTarget?.files?.[0];
    const { updateLocation } = this.props;

    if (file) {
      const reader = new FileReader();
      const readerOnLoad = () => {
        return async (e: any) => {
          this.setState({ isUploading: true });
          try {
            await postLicenseToken(e.target.result);
            updateLocation({ query: { tokenUpdated: true }, partial: true });
            setTimeout(() => {
              this.setState({ isUploading: false });
              // reload from server to pick up the new token
              location.reload();
            }, 1000);
          } catch {
            updateLocation({ query: { tokenUpdated: null }, partial: true });
            this.setState({ isUploading: false });
          }
        };
      };
      reader.onload = readerOnLoad();
      reader.readAsText(file);
    }
  };

  render() {
    const { navModel, tokenUpdated, updateLocation } = this.props;
    const { token, isLoading, isUploading } = this.state;

    let editionNotice = 'You are running Grafana Enterprise without a license. The enterprise features are disabled.';
    if (token && ![VALID, EXPIRED].includes(token.status)) {
      editionNotice = 'There is a problem with your Enterprise license token. The enterprise features are disabled.';
    }

    return (
      <Page navModel={navModel}>
        <Page.Contents isLoading={isLoading}>
          {token && [VALID, EXPIRED].includes(token.status) ? (
            <LicenseChrome header={'Enterprise license details'} subheader={token.company}>
              <LicenceInfo token={token} />
              <div>
                <LicenseTokenUpload
                  title="Update License"
                  onFileUpload={this.onFileUpload}
                  isUploading={isUploading}
                  tokenUpdated={tokenUpdated}
                  onWarningRemove={() => updateLocation({ query: { tokenUpdated: null }, partial: true })}
                />
              </div>
            </LicenseChrome>
          ) : (
            <>
              <UpgradeInfo editionNotice={editionNotice} />
              <div style={{ marginLeft: '79px' }}>
                <LicenseTokenUpload
                  title="Have a License?"
                  onFileUpload={this.onFileUpload}
                  isUploading={isUploading}
                />
              </div>
            </>
          )}
        </Page.Contents>
      </Page>
    );
  }
}

interface LicenseInfoProps {
  token: LicenseToken;
}

const LicenceInfo: React.FC<LicenseInfoProps> = ({ token }) => {
  return (
    <table className="filter-table form-inline">
      <tbody>
        <tr>
          <td style={{ width: '30%' }}>License ID</td>
          <td style={{ width: '70%' }}>
            {token.lid} (
            <a
              href={`${token.iss}/licenses/${token.lid}`}
              target="_blank"
              rel="noopener noreferer"
              aria-label="View details about your license in Grafana Cloud"
            >
              View Details
            </a>
            )
          </td>
        </tr>
        <tr>
          <td>Licensed URL</td>
          <td>
            <a href={token.sub} target="_blank" rel="noopener noreferer">
              {token.sub}
            </a>
          </td>
        </tr>
        <tr>
          <td>Company</td>
          <td>{token.company}</td>
        </tr>
        <tr>
          <td>Products</td>
          <td>{token.prod}</td>
        </tr>
        <tr>
          <td>Max Users</td>
          <td>{token.max_users > 0 ? token.max_users : 'Unlimited'}</td>
        </tr>
        <tr>
          <td>Included Editors & Admins</td>
          <td>{token.included_admins > 0 ? token.included_admins : 'Unlimited'}</td>
        </tr>
        <tr>
          <td>Included Viewers</td>
          <td>{token.included_viewers > 0 ? token.included_viewers : 'Unlimited'}</td>
        </tr>
        <tr>
          <td>License Issued</td>
          <td>{new Date(token.nbf * 1000).toLocaleDateString()}</td>
        </tr>
        <tr>
          <td>License Expires</td>
          <td>{new Date(token.lexp * 1000).toLocaleDateString()}</td>
        </tr>
        <tr>
          <td colSpan={2} className="admin-settings-section">
            Token Details
          </td>
        </tr>
        <tr>
          <td>Token ID</td>
          <td>{token.jti}</td>
        </tr>
        <tr>
          <td>Token Issued</td>
          <td>{new Date(token.iat * 1000).toLocaleDateString()}</td>
        </tr>
        <tr>
          <td>Token Expires</td>
          <td>{new Date(token.exp * 1000).toLocaleDateString()}</td>
        </tr>
      </tbody>
    </table>
  );
};

const mapStateToProps = (state: StoreState) => ({
  navModel: getNavModel(state.navIndex, 'licensing'),
  getLicenseToken: getLicenseToken,
  tokenUpdated: getLocationQuery(state.location).tokenUpdated,
});

const mapDispatchToProps = {
  updateLocation,
};

export default hot(module)(connect(mapStateToProps, mapDispatchToProps)(LicensePage));
