import React from 'react';
import { mount, shallow, ReactWrapper } from 'enzyme';
import AutoSizer from 'react-virtualized-auto-sizer';
import { getTheme } from '@grafana/ui';
import { DashboardModel } from 'app/features/dashboard/state';
import { AnalyticsUsersTab, Props } from './AnalyticsUsersTab';
import { getMockDailySummaries } from '../__mocks__/dailySummariesMocks';
import { getMockRecentUsers } from '../__mocks__/recentUsersMocks';
import { getDashboardUsersInfo, DashboardDailySummaryDTO, UserViewDTO } from '../api';

beforeEach(() => {
  jest.clearAllMocks();
});

jest.mock('../api', () => {
  return {
    getDashboardUsersInfo: jest.fn((dashboardId: number) => {
      const recentUsers = getMockRecentUsers();
      return Promise.resolve({
        creator: dashboardId ? recentUsers[0] : null,
        lastEditor: dashboardId ? recentUsers[1] : null,
      });
    }),
  };
});

const setup = async (
  dashboard: DashboardModel,
  dailySummaries: DashboardDailySummaryDTO[],
  userViews: UserViewDTO[]
): Promise<ReactWrapper> => {
  const props: Props = {
    dashboard,
    dailySummaries,
    userViews,
    theme: getTheme(),
    setDrawerOpen: jest.fn() as any,
    updateLocation: jest.fn() as any,
  };
  let wrapper = await mount(<AnalyticsUsersTab {...props} />);
  return wrapper;
};

describe('Render', () => {
  it('should render empty component - no data', async () => {
    const wrapper = await setup(new DashboardModel({ id: 0 }), [], []);

    expect(wrapper.exists({ 'aria-label': 'User information box' })).toEqual(false);
    expect(wrapper.exists(AutoSizer)).toEqual(false);
  });

  it('should render dashboard meta information', async () => {
    const wrapper = await setup(new DashboardModel({ id: 1 }), [], []);
    expect(getDashboardUsersInfo).toHaveBeenCalledTimes(1);

    wrapper.update();

    expect(wrapper.find({ 'aria-label': 'User information box' })).toHaveLength(2);
    expect(wrapper.exists(AutoSizer)).toEqual(false);
  });

  it('should render views from daily summaries', async () => {
    const wrapper = await setup(new DashboardModel({ id: 0 }), getMockDailySummaries(), []);

    expect(wrapper.exists(AutoSizer)).toEqual(true);
    const autoSizer = shallow(
      wrapper
        .find(AutoSizer)
        .props()
        .children({ width: 100, height: 100 }) as React.ReactElement
    );

    expect(wrapper.exists({ 'aria-label': 'User information box' })).toEqual(false);
    expect(autoSizer.exists({ 'aria-label': 'Graph container' })).toEqual(true);
  });

  it('should render recent users information', async () => {
    const userViews = getMockRecentUsers();
    const wrapper = await setup(new DashboardModel({ id: 0 }, { showSettings: true }), [], userViews);

    expect(wrapper.exists(AutoSizer)).toEqual(true);
    const autoSizer = mount(
      wrapper
        .find(AutoSizer)
        .props()
        .children({ width: 100, height: 100 }) as React.ReactElement
    );

    const userInfoBox = wrapper.find({ 'aria-label': 'User information box' });
    expect(userInfoBox).toHaveLength(1);
    expect(userInfoBox.find({ 'aria-label': 'User box title' }).text()).toEqual('Last viewed');
    expect(wrapper.exists({ 'aria-label': 'Version history button' })).toEqual(true);
    expect(autoSizer.exists({ 'aria-label': 'Recent users table' })).toEqual(true);
    expect(autoSizer.find({ role: 'row' })).toHaveLength(userViews.length + 1);
  });

  it('should not render version history button', async () => {
    const userViews = getMockRecentUsers();
    const wrapper = await setup(new DashboardModel({ id: 0 }, { canEdit: false }), [], userViews);

    expect(wrapper.exists(AutoSizer)).toEqual(true);
    const autoSizer = mount(
      wrapper
        .find(AutoSizer)
        .props()
        .children({ width: 100, height: 100 }) as React.ReactElement
    );

    expect(wrapper.exists({ 'aria-label': 'Version history button' })).toEqual(false);
    expect(autoSizer.exists({ 'aria-label': 'Recent users table' })).toEqual(true);
    expect(autoSizer.find({ role: 'row' })).toHaveLength(userViews.length + 1);
  });
});
